package com.zine.zinemob.scene.controller;

/**
 * Interface para objetos que escutam por eventos de entrada utilizando o teclado
 * do dispositivo.
 */
public interface KeyboardListener {

	/**
	 * Mtodo chamado quando uma tecla  pressionada.
	 * @param keyCode o cdigo da tecla pressionada
	 * @param gameAction a ao vinculada a esta tecla
	 */
	public void onKeyPressed(int keyCode, int gameAction);

	/**
	 * Mtodo chamado quando uma tecla  pressionada e assim  mantida. Este mtodo
	 *  chamado de tempos em tempos enquanto a tecla  mantida pressionada, mas
	 * no em toda a iterao do loop.  levado em considerao um certo tempo de
	 * repetio.
	 * @param keyCode o cdigo da tecla mantida pressionada
	 * @param gameAction a ao vinculada a esta tecla
	 */
	public void onKeyRepeated(int keyCode, int gameAction);

	/**
	 * Mtodo chamado quando uma tecla  solta.
	 * @param keyCode o cdigo da tecla solta
	 * @param gameAction a ao vinculada a esta tecla
	 */
	public void onKeyReleased(int keyCode, int gameAction);

	/**
	 * Mtodo chamado para notificar sobre o estado das teclas pressionadas. 
	 * chamado sempre aps as notificaes de eventos (onKeyPressed, onKeyRepeated
	 * ou onKeyReleased).
	 *
	 * O mtodo no  chamado quando no h nenhuma tecla pressionada.
	 *
	 * @param keyStates valor onde esto armazenadas as teclas pressionadas (game
	 * actions). Este valor contm o mesmo formato que o retornado pelo mtodo
	 * getKeyStates da classe GameCanvas.
	 */
	public void updateKeyStates(int keyStates);

}
